/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.interceptor.description;

import javax.jbi.JBIException;

import org.ow2.petals.component.framework.api.Interceptor;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;

/**
 * The builder of interceptor instances.
 * 
 * @author chamerling - EBM WebSourcing
 * @since 2.0
 * 
 */
public final class InterceptorBuilder {

    /**
     * Creates a new instance of {@link InterceptorBuilder}
     *
     */
    private InterceptorBuilder() {
        // NOP
    }

    /**
     * Load an interceptor instance
     *
     * @return
     */
    public static Interceptor loadInstance(final String className) throws PEtALSCDKException {
        Interceptor interceptor = null;
        final Class<?> clazz = InterceptorBuilder.loadClass(className);

        try {
            final Object o = clazz.newInstance();
            if (o instanceof Interceptor) {
                interceptor = (Interceptor) o;
            } else {
                throw new PEtALSCDKException("Invalid class, " + className
                        + " is not an interceptor");
            }

        } catch (final InstantiationException e) {
            throw new PEtALSCDKException("Can not instanciate interceptor", e);
        } catch (final IllegalAccessException e) {
            throw new PEtALSCDKException("Illegal access to interceptor", e);
        }

        return interceptor;
    }

    /**
     *
     * @param classloader
     * @param className
     * @return
     * @throws JBIException
     */
    protected static Class<?> loadClass(final String className) throws PEtALSCDKException {
        Class<?> clazz;
        final ClassLoader classloader = InterceptorBuilder.class.getClassLoader();
        try {
            clazz = classloader.loadClass(className);
            if (clazz == null) {
                throw new PEtALSCDKException("Could not find interceptor class : " + className);
            }

        } catch (final ClassNotFoundException e) {
            throw new PEtALSCDKException("Class not found interceptor class : " + className, e);
        } catch (final NullPointerException e) {
            throw new PEtALSCDKException("Null class : " + className);
        }
        return clazz;
    }
}
