/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.ow2.petals.component.framework.api.notification.NotificationConsumer;
import org.ow2.petals.component.framework.api.notification.NotificationProducer;
import org.ow2.petals.component.framework.api.notification.Transformer;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Message;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.NotificationMessageHolderType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Notify;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistration;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisher;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.topics.WstopConstants;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * The controller of the broker TODO: convert this to an abstract class with
 * setters and implement it into each component supporting the notifications
 * 
 * @author Frederic Gardes
 * 
 */
public class ComponentNotificationBrokerController implements NotificationBrokerController {

    /** The publisher registration reference provided by the broker. */
    private EndpointReferenceType publisherRegistrationReference = null;

    // TODO use this address to notify
    /**
     * The consumer reference provided by the broker
     */
    private EndpointReferenceType consumerReference = null;

    /**
     * The sender of notification
     */
    private Sender sender = null;

    /**
     * The transformer of notification
     */
    private Transformer transformer = null;

    /**
     * The Notification Producer
     */
    private NotificationProducer notificationProducer = null;

    /**
     * The Notification Consumer
     */
    private NotificationConsumer notificationConsumer = null;

    /**
     * The reader of document to marshal notifications
     */
    private WSNotificationReader wsnReader = null;

    /**
     * The writer of document to unmarshal notifications
     */
    private WSNotificationWriter wsnWriter = null;

    /**
     * The logger
     */
    private Logger logger = null;

    public ComponentNotificationBrokerController(final Logger logger,
            final ComponentContext componentContext) throws WSNotificationException, WSDLException,
            WSAddressingException, WSNotificationExtensionException, MessagingException {
        super();
        this.wsnReader = WSNotificationReader.getInstance();
        this.wsnWriter = WSNotificationWriter.getInstance();
        this.sender = new Sender(componentContext);
        this.logger = logger;
        // initialize the notification producer
        this.notificationProducer = new ComponentNotificationProducer(componentContext
                .getDeliveryChannel(), this.wsnReader, this.wsnWriter, logger);
        // initialize the notification consumer
        this.notificationConsumer = new ComponentNotificationConsumer(componentContext
                .getDeliveryChannel(), this.wsnReader, this.wsnWriter, logger);
        this.transformer = new ComponentTransformer(this.notificationProducer
                .getNotificationProducerSOAParameter(), this.notificationProducer
                .getSubscriptionManagerSOAParameter(), this.notificationConsumer
                .getNotificationConsumerSOAParameter());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#getTransformer()
     */
    public final Transformer getTransformer() {
        return this.transformer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#destroyRegistration()
     */
    public final void destroyRegistration() throws WSNotificationException,
    WSNotificationExtensionException, WSAddressingException, MessagingException,
    PEtALSCDKException {
        this.logger.fine("Destroying the registration...");
        final DestroyRegistration destroyRegistration = WSNotificationFactory.getInstance()
        .createDestroyRegistration();
        final ResourcesUuidType resourcesUuidType = WsnSpecificTypeHelper
        .getResourcesUuidType(this.publisherRegistrationReference.getReferenceParameters());
        WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuidType, destroyRegistration);
        this.sender.send(destroyRegistration);
        this.logger.fine("R egistration destroyed");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#getCurrentMessage()
     */
    public final String getCurrentMessage() throws WSNotificationException, MessagingException,
    PEtALSCDKException, WSAddressingException {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#notify(org.w3c.dom.Document,
     * java.lang.String,
     * com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType
     * )
     */
    public final void notify(final Document document, final String topic,
            final EndpointReferenceType subscriptionReference) throws WSNotificationException,
            MessagingException, PEtALSCDKException, WSAddressingException, WSDLException,
            WSNotificationExtensionException {
        SOAParameterType soaParameter = null;

        final Notify notify = WSNotificationFactory.getInstance().createNotify();
        final NotificationMessageHolderType notificationMessageHolderType = WSNotificationFactory
        .getInstance().createNotificationMessageHolderType();
        // Subscription Reference
        notificationMessageHolderType.setSubscriptionReference(subscriptionReference);
        // Topic
        final TopicExpressionType topicExpressionType = WSNotificationFactory.getInstance()
        .createTopicExpressionType();
        topicExpressionType.addTopicNameSpace("petals", Constants.WSStar.Topic.NAMESPACE_URI);
        topicExpressionType.setDialect(WstopConstants.CONCRETE_TOPIC_EXPRESSION_DIALECT_URI);
        topicExpressionType.setContent(topic);
        notificationMessageHolderType.setTopic(topicExpressionType);
        // Producer Reference
        final EndpointReferenceType producerReference = WSAddressingFactory.getInstance()
        .newEndpointReferenceType();
        final ReferenceParametersType referenceParametersType = producerReference
        .newReferenceParameters();
        soaParameter = this.notificationProducer.getNotificationProducerSOAParameter();
        WsnSpecificTypeHelper.setSOAParameter(soaParameter, referenceParametersType);
        producerReference.setReferenceParameters(referenceParametersType);
        notificationMessageHolderType.setProducerReference(producerReference);
        final Message msg = WSNotificationFactory.getInstance().createMessage();
        WsnSpecificTypeHelper.setContentToMessage(document.getDocumentElement(), msg);
        notificationMessageHolderType.setMessage(msg);
        notify.addNotificationMessage(notificationMessageHolderType);
        this.sender.send(notify, this.consumerReference);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#registerPublisher()
     */
    public final void registerPublisher() throws WSNotificationException, MessagingException,
    PEtALSCDKException, WSAddressingException, WSNotificationExtensionException {
        this.logger.fine("Registering as publisher...");
        RegisterPublisher registerPublisher = WSNotificationFactory.getInstance()
        .createRegisterPublisher();
        // PublisherReference
        registerPublisher = this.transformer.addNotificationProducerSOAParameter(registerPublisher);
        // Topic
        final TopicExpressionType topic = WSNotificationFactory.getInstance()
        .createTopicExpressionType();
        topic.addTopicNameSpace("petals", Constants.WSStar.Topic.NAMESPACE_URI);
        topic.setDialect(WstopConstants.XPATH_TOPIC_EXPRESSION_DIALECT_URI);
        topic.setContent("petals:component/cdk//*[@wstop:topic='true']");
        registerPublisher.addTopic(topic);
        // Demand
        registerPublisher.setDemand(true);
        // InitialTerminationTime: nothing today
        final RegisterPublisherResponse registerPublisherResponse = this.sender
        .send(registerPublisher);
        if (registerPublisherResponse != null) {
            // publisher registration reference
            this.publisherRegistrationReference = registerPublisherResponse
            .getPublisherRegistrationReference();
            // consumer reference
            if (registerPublisherResponse.getConsumerReference() != null) {
                this.consumerReference = registerPublisherResponse.getConsumerReference();
            }
        }
        this.logger.fine("Publisher registered");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController #renew()
     */
    public void renew() {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#subscribe()
     */
    public void subscribe() throws WSNotificationException, MessagingException, PEtALSCDKException,
    WSAddressingException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#unsubscribe()
     */
    public void unsubscribe() throws WSNotificationException, MessagingException,
    PEtALSCDKException, WSAddressingException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationActor
     * #isBrokeredNotification(javax.jbi.messaging.MessageExchange)
     */
    public final boolean isBrokeredNotification(final MessageExchange messageExchange) {
        if (messageExchange == null || messageExchange.getInterfaceName() == null
                || messageExchange.getInterfaceName().getNamespaceURI() == null) {
            return false;
        }
        return Constants.Notification.WS_BROKERED_NOTIFICATION_WSDL_NAMESPACE_URI
        .equals(messageExchange.getInterfaceName().getNamespaceURI());
    }

    /**
     * Get the notificationProducer
     * 
     * @return the notificationProducer
     */
    public final NotificationProducer getNotificationProducer() {
        return this.notificationProducer;
    }

    /**
     * Get the notificationConsumer
     * 
     * @return the notificationConsumer
     */
    public final NotificationConsumer getNotificationConsumer() {
        return this.notificationConsumer;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setNotificationBrokerReferenceAddress(java.lang.String)
     */
    public final void setNotificationBrokerReferenceAddress(
            final String notificationBrokerReferenceAddress) {
        this.transformer.setNotificationBrokerReferenceAddress(notificationBrokerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setPublisherRegistrationManagerReferenceAddress(java.lang.String)
     */
    public final void setPublisherRegistrationManagerReferenceAddress(
            final String publisherRegistrationManagerReferenceAddress) {
        this.transformer
        .setPublisherRegistrationManagerReferenceAddress(publisherRegistrationManagerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setSubscriptionManagerReferenceAddress(java.lang.String)
     */
    public final void setSubscriptionManagerReferenceAddress(
            final String subscriptionManagerReferenceAddress) {
        this.transformer
        .setSubscriptionManagerReferenceAddress(subscriptionManagerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#isInternalNotification(java.lang.String)
     */
    public final boolean isInternalNotification(final String address) {
        if (Constants.Notification.CDK_ADDRESS.equals(address)) {
            return true;
        }
        return false;
    }

    /**
     * Get the wsnReader
     * 
     * @return the wsnReader
     */
    public final WSNotificationReader getWsnReader() {
        return this.wsnReader;
    }

    /**
     * Get the wsnWriter
     * 
     * @return the wsnWriter
     */
    public final WSNotificationWriter getWsnWriter() {
        return this.wsnWriter;
    }
}
