/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.ComponentWsdl;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.Wsdl;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.NotificationController;
import org.ow2.petals.component.framework.api.notification.Transformer;
import org.ow2.petals.component.framework.util.UtilFactory;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistration;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistrationResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisher;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.topics.WstopConstants;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.WSNotificationExtensionFactory;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;
import com.ebmwebsourcing.wsstar.notification.service.basenotification.impl.NotificationProducerMgr;
import com.ebmwebsourcing.wsstar.notification.service.basenotification.impl.SubscriptionManagerMgr;
import com.ebmwebsourcing.wsstar.notification.service.topic.WstopTopicManager;

/**
 * The component controller of notification
 * 
 * @author Frederic Gardes
 * 
 */
/**
 * @author Frederic Gardes
 * 
 */
public class ComponentNotificationController implements NotificationController {
    private NotificationProducerMgr wsnbProducerService = null;

    private SubscriptionManagerMgr wsnbSubscriptionManagerService = null;

    private WstopTopicManager wstopTopicsMgr = null;

    private Logger logger = null;

    private Wsdl producerWsdl = null;

    private Wsdl consumerWsdl = null;

    private Map<String, NotificationProducerReferenceSet> notificationProducerReferenceMap = null;

    /**
     * The transformer of notification
     */
    private Transformer transformer = null;

    /**
     * The sender of notification
     */
    private Sender sender = null;

    /**
     * Create a controller
     * 
     * @param componentContext
     *            The component context, to send exchanges
     * @param logger
     *            The logger
     * @param topicSet
     *            The imput stream to read the topic set TODO create an empty
     *            constructor, for others contexts of creation
     * @throws WSDLException
     * @throws WSNotificationExtensionException
     * @throws WSNotificationException
     * @throws MessagingException
     */
    public ComponentNotificationController(final ComponentContext componentContext,
            final Logger logger, final InputStream topicSet) throws WSDLException,
            WSNotificationExtensionException, WSNotificationException, MessagingException {
        super();
        SOAParameterType subscriptionSOAParameterType = null;
        SOAParameterType producerSOAParameterType = null;
        this.logger = logger;
        this.notificationProducerReferenceMap = new ConcurrentHashMap<String, NotificationProducerReferenceSet>(
                1);
        // the topic manager
        this.wstopTopicsMgr = new WstopTopicManager(topicSet);
        // the producer and subscription manager wsdl
        // TODO activate or no the producer by an external attribute
        this.producerWsdl = new ComponentWsdl(UtilFactory.getWSDLUtil().createWsdlDescription(
                this.getClass().getResource("/" + Constants.Notification.PRODUCER_WSDL), true));
        // the subscription manager
        this.wsnbSubscriptionManagerService = new SubscriptionManagerMgr(logger,
                this.wstopTopicsMgr);
        // TODO add the management of SOAParameterType into the
        // SubscriptionsManager
        subscriptionSOAParameterType = this.getSOAParameter(this.producerWsdl,
                Constants.Notification.SUBSCRIPTION_INTERFACE);
        this.wsnbSubscriptionManagerService
        .setSubscriptionsManagerInterface(subscriptionSOAParameterType.getInterface());
        this.wsnbSubscriptionManagerService
        .setSubscriptionsManagerService(subscriptionSOAParameterType.getService());
        this.wsnbSubscriptionManagerService.setSubscriptionsManagerEdp(subscriptionSOAParameterType
                .getEndpoint());
        // the producer
        // TODO add the management of SOAParameterType into the
        // ProducerManager
        producerSOAParameterType = this.getSOAParameter(this.producerWsdl,
                Constants.Notification.NOTIFICATION_PRODUCER_INTERFACE);
        this.wsnbProducerService = new NotificationProducerMgr(logger, this.wstopTopicsMgr,
                this.wsnbSubscriptionManagerService);
        this.wsnbProducerService.setNotificationProducerInterface(producerSOAParameterType
                .getInterface());
        this.wsnbProducerService.setNotificationProducerService(producerSOAParameterType
                .getService());
        this.wsnbProducerService.setNotificationProducerEdp(producerSOAParameterType.getEndpoint());
        // the consumer wsdl
        // TODO activate or no the consumer by an external attribute
        this.consumerWsdl = new ComponentWsdl(UtilFactory.getWSDLUtil().createWsdlDescription(
                this.getClass().getResource("/" + Constants.Notification.CONSUMER_WSDL), true));
        this.transformer = new ComponentTransformer(producerSOAParameterType,
                subscriptionSOAParameterType, this.getSOAParameter(this.consumerWsdl,
                        Constants.Notification.NOTIFICATION_CONSUMER_INTERFACE));
        this.sender = new Sender(componentContext);
    }

    /**
     * Get the SOA parameter into the wsdl
     * 
     * @param wsdl
     *            The wsdl
     * @param qName
     *            The QName matching with the interface of the service into the
     *            wsdl document
     * @return The SOA parameter
     * @throws WSDLException
     * @throws WSNotificationExtensionException
     * @throws WSNotificationException
     * @throws WSAddressingException
     */
    private SOAParameterType getSOAParameter(final Wsdl wsdl, final QName qName)
    throws WSDLException, WSNotificationExtensionException {
        final List<Endpoint> endpointList = UtilFactory.getWSDLUtil().getEndpointList(
                wsdl.getDescription());
        if (endpointList != null) {
            for (final Endpoint endpoint : endpointList) {
                if (endpoint != null && endpoint.getService() != null
                        && endpoint.getService().getInterface() != null
                        && endpoint.getService().getInterface().getQName() != null) {
                    if (qName.equals(endpoint.getService().getInterface().getQName())) {
                        return this.convertToSOAParameter(endpoint);
                    }
                }
            }
        }
        return null;
    }

    /**
     * Convert a wsdl endpoint to a SOA parameter
     * 
     * @param endpoint
     *            the wsdl endpoint
     * @return The SOA parameter
     * @throws WSNotificationExtensionException
     * @throws WSDLException
     */
    private SOAParameterType convertToSOAParameter(final Endpoint endpoint)
    throws WSNotificationExtensionException, WSDLException {
        SOAParameterType soaParameter = null;

        soaParameter = WSNotificationExtensionFactory.getInstance().createSOAParameterType();
        soaParameter.setInterface(endpoint.getService().getInterface().getQName());
        soaParameter.setService(endpoint.getService().getQName());
        soaParameter.setEndpoint(endpoint.getName());
        return soaParameter;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationController
     * #registerPublisher(java.lang.String, java.lang.String)
     */
    public String registerPublisher(final String topicExpressionNamespace,
            final String topicExpression) throws WSNotificationException, WSAddressingException,
            WSNotificationExtensionException, MessagingException, PEtALSCDKException {

        this.logger.fine("Creating a register publisher...");
        RegisterPublisher registerPublisher = WSNotificationFactory.getInstance()
        .createRegisterPublisher();
        // PublisherReference
        registerPublisher = this.transformer.addNotificationProducerSOAParameter(registerPublisher);
        // Topic
        final TopicExpressionType topic = WSNotificationFactory.getInstance()
        .createTopicExpressionType();
        topic.addTopicNameSpace("petals", topicExpressionNamespace);
        topic.setDialect(WstopConstants.XPATH_TOPIC_EXPRESSION_DIALECT_URI);
        topic.setContent("petals:" + topicExpression);
        registerPublisher.addTopic(topic);
        // Demand
        registerPublisher.setDemand(true);
        // InitialTerminationTime: nothing today
        this.logger.fine("Register publisher created");
        final RegisterPublisherResponse registerPublisherResponse = this
        .registerPublisher(registerPublisher);
        return this.getUuid(registerPublisherResponse);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationController
     * #registerPublisher(com.ebmwebsourcing.wsstar.notification.definition.
     * brokerednotification.api.RegisterPublisher)
     */
    public RegisterPublisherResponse registerPublisher(final RegisterPublisher registerPublisher)
    throws WSNotificationException, WSAddressingException,
    WSNotificationExtensionException, MessagingException, PEtALSCDKException {
        this.logger.fine("Registering as publisher...");
        final RegisterPublisherResponse registerPublisherResponse = this.sender
        .send(registerPublisher);
        final String uuid = this.getUuid(registerPublisherResponse);
        if (uuid != null) {
            // publisher registration reference: mandatory
            final EndpointReferenceType publisherRegistrationReference = registerPublisherResponse
            .getPublisherRegistrationReference();
            if (publisherRegistrationReference != null) {
                final NotificationProducerReferenceSet notificationProducerReferenceSet = new NotificationProducerReferenceSet();
                notificationProducerReferenceSet
                .setPublisherRegistrationReference(publisherRegistrationReference);
                // consumer reference: optional
                if (registerPublisherResponse.getConsumerReference() != null) {
                    final EndpointReferenceType consumerReference = registerPublisherResponse
                    .getConsumerReference();
                    notificationProducerReferenceSet.setConsumerReference(consumerReference);
                }
                // add the registration to the map
                this.notificationProducerReferenceMap.put(uuid, notificationProducerReferenceSet);
                this.logger.fine("Publisher registered");
            }
        }
        return registerPublisherResponse;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationController
     * #destroyRegistration(java.lang.String)
     */
    public void destroyRegistration(final String registrationUuid) throws WSNotificationException,
    WSAddressingException, WSNotificationExtensionException, MessagingException,
    PEtALSCDKException {
        this.logger.fine("Creating a destroy registration...");
        final DestroyRegistration destroyRegistration = WSNotificationFactory.getInstance()
        .createDestroyRegistration();
        final ResourcesUuidType resourcesUuidType = WSNotificationExtensionFactory.getInstance()
        .createResourcesUuidType();
        resourcesUuidType.addUuid(registrationUuid);
        WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuidType, destroyRegistration);
        this.logger.fine("Destroy registration created...");
        this.destroyRegistration(destroyRegistration);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationController
     * #destroyRegistration(com.ebmwebsourcing.wsstar.notification.definition.
     * brokerednotification.api.DestroyRegistration)
     */
    public DestroyRegistrationResponse destroyRegistration(
            final DestroyRegistration destroyRegistration) throws WSNotificationException,
            WSAddressingException, WSNotificationExtensionException, MessagingException,
            PEtALSCDKException {
        DestroyRegistrationResponse destroyRegistrationResponse = null;
        this.logger.fine("Destroying the registration...");
        final String registrationUuid = this.getUuid(destroyRegistration);
        if (registrationUuid != null) {
            final NotificationProducerReferenceSet notificationProducerReferenceSet = this.notificationProducerReferenceMap
            .get(registrationUuid);
            if (notificationProducerReferenceSet != null) {
                final EndpointReferenceType endpointReferenceType = notificationProducerReferenceSet
                .getPublisherRegistrationReference();
                if (endpointReferenceType != null) {
                    final ReferenceParametersType referenceParametersType = endpointReferenceType
                    .getReferenceParameters();
                    if (referenceParametersType != null) {
                        destroyRegistrationResponse = this.sender.send(destroyRegistration,
                                WsnSpecificTypeHelper.getSOAParameter(referenceParametersType));
                        if (destroyRegistrationResponse != null) {
                            this.notificationProducerReferenceMap.remove(registrationUuid);
                            this.logger.fine("Registration destroyed");
                        }
                    }
                }
            }
        }
        return destroyRegistrationResponse;
    }

    public Wsdl getProducerWsdl() {
        return this.producerWsdl;
    }

    public Wsdl getConsumerWsdl() {
        return this.consumerWsdl;
    }

    /**
     * Get the uuid into a register publisher response
     * 
     * @param registerPublisherResponse
     *            The register publisher response
     * @return The uuid into the register publisher response, or null if not
     *         found
     * @throws WSNotificationException
     * @throws WSNotificationExtensionException
     * @throws WSAddressingException
     */
    private String getUuid(final RegisterPublisherResponse registerPublisherResponse)
    throws WSNotificationException, WSNotificationExtensionException, WSAddressingException {
        String uuid = null;

        if (registerPublisherResponse != null) {
            // publisher registration reference
            final EndpointReferenceType publisherRegistrationReference = registerPublisherResponse
            .getPublisherRegistrationReference();
            if (publisherRegistrationReference != null) {
                final ResourcesUuidType resourcesUuidType = WsnSpecificTypeHelper
                .getResourcesUuidType(publisherRegistrationReference
                        .getReferenceParameters());
                if (resourcesUuidType != null) {
                    final List<String> uuidList = resourcesUuidType.getUuids();
                    if (uuidList != null && uuidList.size() == 1) {
                        // uuid
                        uuid = uuidList.get(0);
                    }
                }
            }
        }
        return uuid;
    }

    /**
     * Get the uuid into a destroy registration
     * 
     * @param destroyRegistration
     *            The destroy registration
     * @return The uuid into the destroy registration, or null if not found
     * @throws WSNotificationException
     * @throws WSNotificationExtensionException
     * @throws WSAddressingException
     */
    private String getUuid(final DestroyRegistration destroyRegistration)
    throws WSNotificationException, WSNotificationExtensionException, WSAddressingException {
        String uuid = null;

        if (destroyRegistration != null) {
            final ResourcesUuidType resourcesUuidType = WsnSpecificTypeHelper
            .getResourcesUuidType(destroyRegistration);
            if (resourcesUuidType != null) {
                final List<String> uuidList = resourcesUuidType.getUuids();
                if (uuidList != null && uuidList.size() == 1) {
                    // uuid
                    uuid = uuidList.get(0);
                }
            }
        }
        return uuid;
    }
}
