/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Notify;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Unsubscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.UnsubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.WsnbrConstants;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistration;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistrationResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisher;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * Sender a notification to the bus
 * 
 * @author Frederic Gardes
 * 
 */
public class Sender {

    private ComponentContext componentContext = null;

    /**
     * @deprecated notifications without managers
     * @use the generic factory, for the notifications with managers
     */
    @Deprecated
    private MessageExchangeFactory notificationBrokerMessageExchangeFactory = null;

    /**
     * @deprecated notifications without managers
     * @use the generic factory, for the notifications with managers
     */
    @Deprecated
    private MessageExchangeFactory publisherRegistrationMessageExchangeFactory = null;

    /**
     * @deprecated notifications without managers
     * @use the generic factory, for the notifications with managers
     */
    @Deprecated
    private MessageExchangeFactory subscriptionManagerMessageExchangeFactory = null;

    /**
     * The generic message exchange factory
     */
    private MessageExchangeFactory messageExchangeFactory = null;

    public Sender(final ComponentContext componentContext) throws WSNotificationException,
    MessagingException {
        super();
        this.componentContext = componentContext;
        this.notificationBrokerMessageExchangeFactory = componentContext.getDeliveryChannel()
        .createExchangeFactory(Constants.Notification.NOTIFICATION_BROKER_INTERFACE);
        this.publisherRegistrationMessageExchangeFactory = componentContext.getDeliveryChannel()
        .createExchangeFactory(
                Constants.Notification.PUBLISHER_REGISTRATION_MANAGER_INTERFACE);
        this.subscriptionManagerMessageExchangeFactory = componentContext.getDeliveryChannel()
        .createExchangeFactory(Constants.Notification.SUBSCRIPTION_INTERFACE);
        this.messageExchangeFactory = componentContext.getDeliveryChannel().createExchangeFactory();
    }

    /**
     * @deprecated notifications without managers
     * @use send(final DestroyRegistration destroyRegistration, final
     *      SOAParameterType soaParameterType)
     */
    @Deprecated
    public DestroyRegistrationResponse send(final DestroyRegistration destroyRegistration)
    throws MessagingException, WSNotificationException, PEtALSCDKException {
        DestroyRegistrationResponse destroyRegistrationResponse = null;
        // convert destroyRegistration to document
        Document document = WSNotificationWriter.getInstance().writeDestroyRegistration(
                destroyRegistration);
        document = this.send(document, this.publisherRegistrationMessageExchangeFactory
                .createInOutExchange(), WsnbrConstants.DESTROY_REGISTRATION_QNAME, true);
        if (document != null) {
            // Convert document to destroyRegistration response
            destroyRegistrationResponse = WSNotificationReader.getInstance()
            .readDestroyRegistrationResponse(document);
        }
        return destroyRegistrationResponse;
    }

    public DestroyRegistrationResponse send(final DestroyRegistration destroyRegistration,
            final SOAParameterType soaParameterType) throws MessagingException,
            WSNotificationException, PEtALSCDKException {
        DestroyRegistrationResponse destroyRegistrationResponse = null;
        // convert destroyRegistration to document
        Document document = WSNotificationWriter.getInstance().writeDestroyRegistration(
                destroyRegistration);
        // create the exchange
        final MessageExchange messageExchange = this.messageExchangeFactory.createInOutExchange();
        // TODO implements the way with interface, interface/service,
        // interface/service/endpoint, remove the way service/endpoint
        final ServiceEndpoint serviceEndpoint = this.componentContext.getEndpoint(soaParameterType
                .getService(), soaParameterType.getEndpoint());
        if (serviceEndpoint != null) {
            messageExchange.setEndpoint(serviceEndpoint);
            document = this.send(document, messageExchange,
                    WsnbrConstants.DESTROY_REGISTRATION_QNAME, true);
            if (document != null) {
                // Convert document to destroyRegistration response
                destroyRegistrationResponse = WSNotificationReader.getInstance()
                .readDestroyRegistrationResponse(document);
            }
        }
        return destroyRegistrationResponse;
    }

    private Document send(final Document notification, final MessageExchange messageExchange,
            final QName operation, final boolean synchronous) throws MessagingException,
            PEtALSCDKException {
        NormalizedMessage normalizedMessage = null;
        Source source = null;
        normalizedMessage = messageExchange.createMessage();
        // set in message
        notification.normalizeDocument();
        source = UtilFactory.getSourceUtil().createDOMSource(notification);
        normalizedMessage.setContent(source);
        messageExchange.setMessage(normalizedMessage, "in");
        messageExchange.setOperation(operation);
        if (!synchronous) {
            this.componentContext.getDeliveryChannel().send(messageExchange);
        } else if (this.componentContext.getDeliveryChannel().sendSync(messageExchange)) {
            if (ExchangeStatus.ERROR.equals(messageExchange.getStatus())) {
                throw new MessagingException(messageExchange.getError());
            } else if (ExchangeStatus.ACTIVE.equals(messageExchange.getStatus())) {
                normalizedMessage = messageExchange.getFault();
                if (normalizedMessage == null) {
                    // get out message
                    normalizedMessage = messageExchange.getMessage("out");
                }
                // close the sent exchange
                messageExchange.setStatus(ExchangeStatus.DONE);
                this.componentContext.getDeliveryChannel().send(messageExchange);
                if (normalizedMessage != null) {
                    source = normalizedMessage.getContent();
                    if (source != null) {
                        return UtilFactory.getSourceUtil().createDocument(source);
                    }
                }
            }
        }
        return null;
    }

    public RegisterPublisherResponse send(final RegisterPublisher registerPublisher)
    throws MessagingException, WSNotificationException, PEtALSCDKException {
        RegisterPublisherResponse registerPublisherResponse = null;
        // convert registerPublisher to document
        Document document = WSNotificationWriter.getInstance().writeRegisterPublisher(
                registerPublisher);
        document = this.send(document, this.notificationBrokerMessageExchangeFactory
                .createInOutExchange(), WsnbrConstants.REGISTER_PUBLISHER_QNAME, true);
        if (document != null) {
            registerPublisherResponse = WSNotificationReader.getInstance()
            .readRegisterPublisherResponse(document);
        }
        return registerPublisherResponse;
    }

    public SubscribeResponse send(final Subscribe subscribe) throws MessagingException,
    WSNotificationException, PEtALSCDKException {
        SubscribeResponse subscribeResponse = null;
        // convert subscribe to document
        Document document = WSNotificationWriter.getInstance().writeSubscribe(subscribe);
        document = this.send(document, this.notificationBrokerMessageExchangeFactory
                .createInOutExchange(), WsnbConstants.SUBSCRIBE_QNAME, true);
        if (document != null) {
            // Convert document to subscribe response
            subscribeResponse = WSNotificationReader.getInstance().readSubscribeResponse(document);
        }
        return subscribeResponse;
    }

    public UnsubscribeResponse send(final Unsubscribe unsubscribe) throws MessagingException,
    WSNotificationException, PEtALSCDKException {
        UnsubscribeResponse unsubscribeResponse = null;
        // convert unsubscribe to document
        Document document = WSNotificationWriter.getInstance().writeUnsubscribe(unsubscribe);
        document = this.send(document, this.subscriptionManagerMessageExchangeFactory
                .createInOutExchange(), WsnbConstants.UNSUBSCRIBE_QNAME, true);
        if (document != null) {
            // Convert document to unsubscribe response
            unsubscribeResponse = WSNotificationReader.getInstance().readUnsubscribeResponse(
                    document);
        }
        return unsubscribeResponse;
    }

    /**
     * Send a notify on a consumerReference
     * 
     * @param notify
     *            The notify to send
     * @param consumerReference
     *            The reference to send to
     * @throws WSNotificationExtensionException
     * @throws WSAddressingException
     * @throws WSNotificationException
     * @throws MessagingException
     * @throws PEtALSCDKException
     */
    public void send(final Notify notify, final EndpointReferenceType consumerReference)
    throws WSAddressingException, WSNotificationExtensionException,
    WSNotificationException, MessagingException, PEtALSCDKException {
        SOAParameterType soaParameter = null;

        final InOnly inOnly = this.notificationBrokerMessageExchangeFactory.createInOnlyExchange();
        if (consumerReference != null && consumerReference.getReferenceParameters() != null) {
            soaParameter = WsnSpecificTypeHelper.getSOAParameter(consumerReference
                    .getReferenceParameters());
            if (soaParameter != null) {
                final List<QName> interfaceList = new ArrayList<QName>(1);
                interfaceList.add(soaParameter.getInterface());
                final ServiceEndpoint serviceEndpoint = new NotificationServiceEndpoint(
                        interfaceList, soaParameter.getService(), soaParameter.getEndpoint());
                inOnly.setEndpoint(serviceEndpoint);
            }
        }
        final Document document = WSNotificationWriter.getInstance().writeNotify(notify);
        this.send(document, inOnly, WsnbConstants.NOTIFY_QNAME, false);
    }
}
