/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.filter.messagecontent;

import java.util.List;

import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.component.framework.notification.filter.Filter;

import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.FilterType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.QueryExpressionType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * A filter on the interface, service and/or endpoint of the exchange
 * 
 * @author Frederic Gardes
 * 
 */
public class EndpointFilter implements Filter {
    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.notification.filter.Filter#isNotifiable
     * (com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.
     * FilterType, java.lang.Object)
     */
    public boolean isNotifiable(FilterType filterType, Object content) {
        ServiceEndpoint serviceEndpoint = null;

        try {
            // TODO not test all the MessageContent
            List<QueryExpressionType> queryExpressionList = filterType.getMessageContentList();
            for (QueryExpressionType queryExpression : queryExpressionList) {
                SOAParameterType soaParameter = WsnSpecificTypeHelper
                        .getSOAParameter(queryExpression);
                if (soaParameter != null) {
                    if (content == null || !(content instanceof ServiceEndpoint)) {
                        return false;
                    }
                    serviceEndpoint = (ServiceEndpoint) content;
                    if (soaParameter.getInterface() != null
                            && !soaParameter.getInterface().equals(
                                    serviceEndpoint.getInterfaces()[0])) {
                        return false;
                    } else if (soaParameter.getService() != null
                            && !soaParameter.getService().equals(serviceEndpoint.getServiceName())) {
                        return false;
                    } else if (soaParameter.getEndpoint() != null
                            && !soaParameter.getEndpoint()
                                    .equals(serviceEndpoint.getEndpointName())) {
                        return false;
                    }
                }
            }
        } catch (WSNotificationExtensionException e) {
            return false;
        } catch (WSNotificationException e) {
            return false;
        }
        return true;
    }

}
