/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import java.util.Map;
import java.util.Set;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.notification.filter.messagecontent.EndpointFilter;
import org.ow2.petals.component.framework.notification.filter.messagecontent.XPathFilter;
import org.ow2.petals.component.framework.notification.filter.topicexpression.TopicExpressionFilter;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.ContextSubscriptionPolicy;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.ProcessSubscriptionPolicy;
import org.ow2.petals.component.framework.notification.subscriptionpolicy.XSLTSubscriptionPolicy;
import org.w3c.dom.Document;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;

/**
 * The abstract notification of a message on a JBI Producer
 * 
 * @author Frederic Gardes
 * 
 */
public abstract class AbstractProducerComponentNotify extends ComponentNotify {
    /**
     * @param abstractComponent
     */
    public AbstractProducerComponentNotify(final AbstractComponent abstractComponent) {
        super(abstractComponent);
    }

    /**
     * Process the notify
     * 
     * @param exchange
     *            The exchange
     * @param document
     *            The document to analyze
     * @param status
     *            The status of the notify
     */
    protected void process(final Exchange exchange, Document document, final int status) {
        TopicExpressionFilter topicExpressionFilter = null;
        XPathFilter xPathFilter = null;
        EndpointFilter endpointFilter = null;
        XSLTSubscriptionPolicy xsltSubscriptionPolicy = null;
        ContextSubscriptionPolicy contextSubscriptionPolicy = null;
        ProcessSubscriptionPolicy processSubscriptionPolicy = null;
        boolean notify = true;
        Subscribe subscribe = null;

        try {
            if (exchange.isProviderRole()) {
                Set<Map.Entry<EndpointReferenceType, Subscribe>> subscriptionEntrySet = null;
		if(this.notificationBrokerController != null) {
			subscriptionEntrySet = this.notificationBrokerController.getNotificationProducer().getSubscriptionMap().entrySet();
		}
                if (subscriptionEntrySet != null) {
                    for (final Map.Entry<EndpointReferenceType, Subscribe> subscriptionEntry : subscriptionEntrySet) {
                        if (subscriptionEntry != null) {
                            subscribe = subscriptionEntry.getValue();
                            if (subscribe != null) {
                                if (subscribe.getFilter() != null) {
                                    // check filter
                                    // topic expression
                                    if (topicExpressionFilter == null) {
                                        topicExpressionFilter = new TopicExpressionFilter();
                                    }
                                    notify = topicExpressionFilter.isNotifiable(subscribe.getFilter(),
                                            this.getTopic());
                                    if (notify) {
                                        // message content
                                        if (xPathFilter == null) {
                                            xPathFilter = new XPathFilter();
                                        }
                                        notify = xPathFilter.isNotifiable(subscribe.getFilter(),
                                                document);
                                        if (notify) {
                                            if (endpointFilter == null) {
                                                endpointFilter = new EndpointFilter();
                                            }
                                            notify = endpointFilter.isNotifiable(subscribe.getFilter(),
                                                    exchange.getEndpoint());
                                            if (notify) {
                                                // producer properties
                                            }
                                        }
                                    }
                                }
                                if (notify) {
                                    // apply subscription policies
                                    if (subscribe.getSubscriptionPolicy() != null) {
                                        // xslt
                                        if (xsltSubscriptionPolicy == null) {
                                            xsltSubscriptionPolicy = new XSLTSubscriptionPolicy(
                                                    this.logger);
                                        }
                                        document = xsltSubscriptionPolicy.applyPolicy(subscribe
                                                .getSubscriptionPolicy(), document);
                                        // context
                                        if (contextSubscriptionPolicy == null) {
                                            contextSubscriptionPolicy = new ContextSubscriptionPolicy(
                                                    exchange, status);
                                        }
                                        document = contextSubscriptionPolicy.applyPolicy(subscribe
                                                .getSubscriptionPolicy(), document);
                                        // process
                                        if (processSubscriptionPolicy == null) {
                                            processSubscriptionPolicy = new ProcessSubscriptionPolicy(
                                                    exchange);
                                        }
                                        document = processSubscriptionPolicy.applyPolicy(subscribe
                                                .getSubscriptionPolicy(), document);
                                    }
                                    this.process(document, this.getTopic(), subscriptionEntry.getKey());
                                }
                            }
                        }
                    }
                }
            }
        } catch (final WSNotificationException e) {
            this.logger.warning("Unable to create the notify: " + e.getMessage());
        }
    }

    /**
     * Get the topic
     * 
     * @return the topic
     */
    public abstract String getTopic();

    /**
     * Process the notify
     * 
     * @param exchange
     *            The exchange
     * @return an exchange not modified
     * @throws MessagingException
     */
    public abstract Exchange process(Exchange exchange) throws MessagingException;
}
