/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.ow2.petals.component.framework.api.notification.generation.Notify;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;

/**
 * The notification
 * 
 * @author Frederic Gardes
 * 
 */
/**
 * @author Frederic Gardes
 * 
 */
public class ComponentNotify implements Notify {

    protected NotificationBrokerController notificationBrokerController = null;

    protected Logger logger = null;

    public ComponentNotify(final AbstractComponent abstractComponent) {
        super();
        notificationBrokerController = abstractComponent.getNotificationBrokerController();
        logger = abstractComponent.getLogger();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.generation.Notify
     * #process(org.w3c.dom.Document, java.lang.String,
     * com.ebmwebsourcing.wsstar
     * .addressing.definition.api.EndpointReferenceType)
     */
    public void process(final Document document, final String topic,
            final EndpointReferenceType subscriptionReference) {
        if (notificationBrokerController != null) {
            try {
                notificationBrokerController.notify(document, topic, subscriptionReference);
            } catch (final WSNotificationException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            } catch (final WSAddressingException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            } catch (final WSDLException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            } catch (final PEtALSCDKException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            } catch (final MessagingException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            } catch (final WSNotificationExtensionException e) {
                logger.info("Unable to send a notification");
                logger.fine("Detail on the notification error: " + e.getMessage());
            }
        }
    }
}
