/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import javax.jbi.messaging.MessagingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * The notification of an exchange with a Done or Error status on a JBI Producer
 * 
 * @author Frederic Gardes
 * 
 */
public class StatusProducerComponentNotify extends AbstractProducerComponentNotify {

    /**
     * @param abstractComponent
     */
    public StatusProducerComponentNotify(final AbstractComponent abstractComponent) {
        super(abstractComponent);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.generation.
     * AbstractProducerComponentNotify#getTopic()
     */
    @Override
    public String getTopic() {
        return "petals:component/cdk/producer/status[@wstop:topic='true']";
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.generation.
     * AbstractProducerComponentNotify
     * #process(org.ow2.petals.component.framework.api.message.Exchange)
     */
    @Override
    public Exchange process(final Exchange exchange) throws MessagingException {
        Document document = null;
        Element root = null;
        int status;
        try {
            if (exchange.isErrorStatus() || exchange.isDoneStatus()) {
                // TODO create a DocumentUtil, static singleton to do this only
                // one time
                final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
                .newInstance();
                final DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
                document = documentBuilder.newDocument();
                if (document != null) {
                    if (exchange.isErrorStatus()) {
                        root = document.createElement("exception");
                        if (exchange.getError() == null || exchange.getError().getMessage() == null) {
                            root.setTextContent("An unknown exception append");
                        } else {
                            root.setTextContent(exchange.getError().getMessage());
                        }
                        status = 1;
                    } else {
                        root = document.createElement("status");
                        status = 0;
                    }
                    document.appendChild(root);
                    process(exchange, document, status);
                }
                documentBuilder.reset();
            }
        } catch (final ParserConfigurationException e) {
            logger.info("Unable to send a notification");
            logger.fine("Detail on the notification error: " + e.getMessage());
        }
        return exchange;
    }

}
