/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.subscriptionpolicy;

import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscriptionPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ContextPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * A subscription policy to add informations into the initial document: <li>the
 * endpoint called by the exchange</li> <li>the service called by the exchange</li>
 * <li>the interface called by the exchange</li> <li>the exchange id</li> <li>
 * the date of the notification</li> <li>the technical correlation id of the
 * flow</li> <li>the status</li>
 * 
 * @author Frederic Gardes
 * 
 */
public class ContextSubscriptionPolicy implements SubscriptionPolicy {

    private Exchange exchange = null;

    private final int status;

    public ContextSubscriptionPolicy(final Exchange exchange, final int status) {
        super();
        this.exchange = exchange;
        this.status = status;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.subscriptionpolicy.
     * SubscriptionPolicy
     * #applyPolicy(com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.SubscriptionPolicyType, org.w3c.dom.Document)
     */
    public Document applyPolicy(final SubscriptionPolicyType subscriptionPolicyType, final Document document) {
        try {
            final ContextPolicyType contextPolicy = WsnSpecificTypeHelper
            .getContextPolicyType(subscriptionPolicyType);
            if (contextPolicy != null) {
                final Element root = document.getDocumentElement();
                if (root != null) {

                    if (contextPolicy.isInterface()) {
                        root.setAttribute("interface", exchange.getEndpoint().getInterfaces()[0]
                                                                                              .toString());
                    }

                    if (contextPolicy.isService()) {
                        root.setAttribute("service", exchange.getEndpoint().getServiceName()
                                .toString());
                    }
                    if (contextPolicy.isEndpoint()) {
                        root.setAttribute("endpoint", exchange.getEndpoint().getEndpointName());
                    }
                    if (contextPolicy.isMeuuid()) {
                        root.setAttribute("meuuid", exchange.getExchangeId());
                    }
                    if (contextPolicy.isNotifDate()) {
                        root.setAttribute("notifDate", Long.toString(System.currentTimeMillis()));
                    }
                    if (contextPolicy.isCorrelationId()
                            && exchange.getProperty(
                                    Message.CORRELATION_ID_KEY) != null
                                    && exchange.getProperty(
                                            Message.CORRELATION_ID_KEY) instanceof String) {
                        final String correlationId = (String) exchange
                                .getProperty(
                                Message.CORRELATION_ID_KEY);
                        root.setAttribute("correlationId", correlationId);
                    }
                    if (contextPolicy.isStatus()) {
                        root.setAttribute("status", Integer.toString(status));
                    }
                }
            }
        } catch (final WSNotificationExtensionException e) {
            // nothing to do
        }
        return document;
    }

}
