/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.subscriptionpolicy;

import java.util.List;

import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscriptionPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ProcessPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;
import com.ebmwebsourcing.wsstar.notification.jaxbextension.types.ActionType;

/**
 * A subscription policy to add a process path into the initial document, and
 * store or remove it into the exchange. The type define the actions: <li>
 * create: if no process path exists, store a new one into the exchange</li> <li>
 * reset: always store a new process path into the exchange</li> <li>destroy:
 * always destroy the process path into the exchange</li> <li>push: if a process
 * path exists into the exchange, add a sub level to it</li> <li>pop: if a
 * process path with at least one sub level exists into the exchange, drop the
 * last sub level to it</li>
 * 
 * @author Frederic Gardes
 * 
 */
public class ProcessSubscriptionPolicy implements SubscriptionPolicy {

    private final static String SEPARATOR = "/";

    private Exchange exchange = null;

    public ProcessSubscriptionPolicy(final Exchange exchange) {
        super();
        this.exchange = exchange;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.subscriptionpolicy.
     * SubscriptionPolicy
     * #applyPolicy(com.ebmwebsourcing.wsstar.notification.definition
     * .basenotification.api.SubscriptionPolicyType, org.w3c.dom.Document)
     */
    public Document applyPolicy(final SubscriptionPolicyType subscriptionPolicyType,
            final Document document) {
        String processPath = null;

        try {
            final ProcessPolicyType processPolicy = WsnSpecificTypeHelper
            .getProcessPolicyType(subscriptionPolicyType);
            if (processPolicy != null && processPolicy.getAction() != null
                    && !processPolicy.getAction().isEmpty()) {
                final List<ActionType> actionList = processPolicy.getAction();
                for (final ActionType action : actionList) {
                    if (action != null) {
                        if (this.exchange.getProperty(Message.PROCESS_PATH_KEY) != null
                                && this.exchange.getProperty(Message.PROCESS_PATH_KEY) instanceof String) {
                            processPath = (String) this.exchange
                            .getProperty(Message.PROCESS_PATH_KEY);
                        }
                        if (processPath == null) {
                            if (ActionType.CREATE.equals(action)) {
                                processPath = UtilFactory.getIdUtil().createId();
                                this.exchange.setProperty(Message.PROCESS_PATH_KEY, processPath);
                            }
                        } else if (ActionType.RESET.equals(action)) {
                            processPath = UtilFactory.getIdUtil().createId();
                            this.exchange.setProperty(Message.PROCESS_PATH_KEY, processPath);
                        } else if (ActionType.DESTROY.equals(action)) {
                            this.exchange.setProperty(Message.PROCESS_PATH_KEY, null);
                        } else if (ActionType.PUSH.equals(action)) {
                            processPath += SEPARATOR + UtilFactory.getIdUtil().createId();
                            this.exchange.setProperty(Message.PROCESS_PATH_KEY, processPath);
                        } else if (ActionType.POP.equals(action)) {
                            if (processPath.lastIndexOf(SEPARATOR) != -1) {
                                processPath = processPath.substring(0, processPath
                                        .lastIndexOf(SEPARATOR));
                                this.exchange.setProperty(Message.PROCESS_PATH_KEY, processPath);
                            }
                        }
                    }
                }
            }
            final Element root = document.getDocumentElement();
            if (root != null && processPath != null) {
                root.setAttribute("processPath", processPath);
            }
        } catch (final WSNotificationExtensionException e) {
            // nothing to do
        }
        return document;
    }

}
