/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.util;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.api.util.ExchangeUtil;
import org.ow2.petals.component.framework.api.util.IdUtil;
import org.ow2.petals.component.framework.api.util.SourceUtil;
import org.ow2.petals.component.framework.api.util.WSDLUtil;
import org.ow2.petals.component.framework.api.util.XSLTUtil;

/**
 * The factory to create one shared instance of each util, and exposes them into
 * the API
 * 
 * @author Frederic Gardes
 * 
 */
public class UtilFactory {
    private static ExchangeUtil exchangeUtil = null;

    private static WSDLUtil wsdlUtil = null;

    private static XSLTUtil xsltUtil = null;

    private static SourceUtil sourceUtil = null;

    private static IdUtil idUtil = null;

    private static WSDLException wsdlUtilException = null;

    static {
        exchangeUtil = new ExchangeUtilImpl();
        sourceUtil = new SourceUtilImpl();
        xsltUtil = new XSLTUtilImpl();
        idUtil = new IdUtilImpl();
        try {
            wsdlUtil = new WSDLUtilImpl();
        } catch (WSDLException e) {
            wsdlUtilException = e;
        }
    }

    /**
     * Get the singleton instance of the ExchangeUtil
     * 
     * @return an ExchangeUtil
     */
    public static ExchangeUtil getExchangeUtil() {
        return exchangeUtil;
    }

    /**
     * Get the singleton instance of the WSDLUtil
     * 
     * @return a WSDLUtil
     * @throws WSDLException
     */
    public static WSDLUtil getWSDLUtil() throws WSDLException {
        if (wsdlUtil == null) {
            throw wsdlUtilException;
        }
        return wsdlUtil;
    }

    /**
     * Get the singleton instance of the SourceUtil
     * 
     * @return a SourceUtil
     */
    public static SourceUtil getSourceUtil() {
        return sourceUtil;
    }

    /**
     * Get the singleton instance of the XSLTUtil
     * 
     * @return a xsltUtil
     */
    public static XSLTUtil getXSLTUtil() {
        return xsltUtil;
    }

    /**
     * Get the singleton instance of the IdUtil
     * 
     * @return an idUtil
     */
    public static IdUtil getIdUtil() {
        return idUtil;
    }
}
