/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.util;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.api.util.XSLTUtil;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * This class provides a set of methods to load and transform XML documents. It
 * supports parameter-aware stylesheets (XSLT).
 * 
 * @author Frederic Gardes
 * 
 */
public class XSLTUtilImpl implements XSLTUtil {

    /**
     * The constructor
     */
    public XSLTUtilImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#loadXML(java.lang.String
     * )
     */
    public Document loadXML(String filename) throws IOException, ParserConfigurationException,
            SAXException {
        DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        return builder.parse(new File(filename));
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocument(org
     * .w3c.dom.Document, java.lang.String)
     */
    public Document transformDocument(Document xmlDocument, String xsltFilename)
            throws TransformerException, ParserConfigurationException {
        return transformDocument(xmlDocument, null, xsltFilename);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocument(org
     * .w3c.dom.Document, javax.xml.transform.Source)
     */
    public Document transformDocument(Document xmlDocument, Source xsltSource)
            throws TransformerException, ParserConfigurationException {
        return transformDocument(xmlDocument, null, xsltSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.XSLTUtil#transformDocumentAsString
     * (org.w3c.dom.Document, java.lang.String)
     */
    public String transformDocumentAsString(Document xmlDocument, String xsltFilename)
            throws TransformerFactoryConfigurationError, TransformerException {
        return transformDocumentAsString(xmlDocument, null, xsltFilename);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document,
     * javax.xml.transform.Source)
     */
    public String transformDocumentAsString(Document xmlDocument, Source xsltSource)
            throws TransformerFactoryConfigurationError, TransformerException {
        return transformDocumentAsString(xmlDocument, null, xsltSource);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#transformDocument
     * (org.w3c.dom.Document, java.util.Hashtable, java.lang.String)
     */
    public Document transformDocument(Document xmlDocument, Hashtable<String, String> parameters,
            Source xsltSource) throws TransformerException, ParserConfigurationException {
        Transformer transformer = createTransformer(parameters, xslNormalize(xsltSource));
        // Create an empty DOMResult object for the output.
        DocumentBuilderFactory dFactory = DocumentBuilderFactory.newInstance();
        dFactory.setNamespaceAware(true);
        DocumentBuilder dBuilder = dFactory.newDocumentBuilder();
        Document dstDocument = dBuilder.newDocument();
        DOMResult domResult = new DOMResult(dstDocument);
        // Perform the transformation.
        transformer.transform(xslNormalizeAsSource(xmlDocument), domResult);
        transformer.reset();
        // Now you can get the output Node from the DOMResult.
        return dstDocument;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.XSLTUtil#transformDocument
     * (org.w3c.dom.Document, java.util.Hashtable, java.lang.String)
     */
    public Document transformDocument(Document xmlDocument, Hashtable<String, String> parameters,
            String xsltFilename) throws TransformerException, ParserConfigurationException {
        Transformer transformer = createTransformer(parameters, xslNormalize(new StreamSource(
                xsltFilename)));
        // Create an empty DOMResult object for the output.
        DocumentBuilderFactory dFactory = DocumentBuilderFactory.newInstance();
        dFactory.setNamespaceAware(true);
        DocumentBuilder dBuilder = dFactory.newDocumentBuilder();
        Document dstDocument = dBuilder.newDocument();
        DOMResult domResult = new DOMResult(dstDocument);
        // Perform the transformation.
        transformer.transform(xslNormalizeAsSource(xmlDocument), domResult);
        transformer.reset();
        // Now you can get the output Node from the DOMResult.
        return dstDocument;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document, java.util.Hashtable,
     * java.lang.String)
     */
    public String transformDocumentAsString(Document xmlDocument,
            Hashtable<String, String> parameters, String xsltFilename)
            throws TransformerFactoryConfigurationError, TransformerException {
        return transformDocumentAsString(xmlDocument, parameters, new StreamSource(xsltFilename));
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.util.XSLTUtil#
     * transformDocumentAsString(org.w3c.dom.Document, java.util.Hashtable,
     * javax.xml.transform.Source)
     */
    public String transformDocumentAsString(Document xmlDocument,
            Hashtable<String, String> parameters, Source xsltSource)
            throws TransformerFactoryConfigurationError, TransformerException {
        Transformer transformer = createTransformer(parameters, xslNormalize(xsltSource));
        StringWriter stringWriter = new StringWriter();
        StreamResult streamResult = new StreamResult(stringWriter);
        // Perform the transformation.
        transformer.transform(xslNormalizeAsSource(xmlDocument), streamResult);
        transformer.reset();
        // Now you can get the output Node from the DOMResult.
        return stringWriter.toString();
    }

    /**
     * Create the transformer
     * 
     * @param parameters
     *            the hashtable with the parameters to be passed to the
     *            stylesheet
     * @param xsltSource
     *            the source of the stylesheet
     * @return the transformer
     * @throws TransformerFactoryConfigurationError
     * @throws TransformerException
     */
    private Transformer createTransformer(Hashtable<String, String> parameters, Source xsltSource)
            throws TransformerFactoryConfigurationError, TransformerException {
        Transformer transformer = null;
        TransformerFactory transformerFactory = null;
        // Generate a Transformer.
        transformerFactory = TransformerFactory.newInstance();
        transformer = transformerFactory.newTransformer(xsltSource);
        // set transformation parameters
        if (parameters != null) {
            Enumeration<String> keys = parameters.keys();
            while (keys.hasMoreElements()) {
                String key = keys.nextElement();
                String value = parameters.get(key);
                transformer.setParameter(key, value);
            }
        }
        return transformer;
    }

    /**
     * Normalize a document for the xsl engine
     * 
     * @param document
     *            The document to normalize
     * @return a document normalized
     * @throws TransformerException
     */
    private Document xslNormalize(Document document) throws TransformerException {
        // Generate a Transformer.
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        DOMResult dr = new DOMResult();
        DOMSource source = new DOMSource(document);
        transformer.transform(source, dr);
        transformer.reset();
        transformer = null;
        return (Document) dr.getNode();
    }

    /**
     * Normalize a document for the xsl engine
     * 
     * @param source
     *            The source to normalize
     * @return a source normalized
     * @throws TransformerException
     */
    private Source xslNormalize(Source source) throws TransformerException {
        // Generate a Transformer.
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        DOMResult dr = new DOMResult();
        transformer.transform(source, dr);
        transformer.reset();
        Document newDoc = (Document) dr.getNode();
        return new DOMSource(newDoc);
    }

    /**
     * Normalize a document for the xsl engine and return as Source
     * 
     * @param document
     *            The document to normalize
     * @return a source normalized
     * @throws TransformerException
     */
    private Source xslNormalizeAsSource(Document document) throws TransformerException {
        Document newDoc = xslNormalize(document);
        return new DOMSource(newDoc);
    }
}
